/* Rocket mass evaluation, computes a single stage and air-launched (balloon or
 * aircraft) rocket's mass given several parameters.
 * This is version 3. The file is documented at:
 * http://lcas.otaski.org/index.php/File:Rocket_mass.c
 * Copyright 2012-, OTASKI (http://otaski.org), licence is BSD
 */
#include <stdio.h>
#include <stdlib.h>	// atof
#include <math.h>	// cbrt, sqrt
#include <string.h>	// strcmp

#define G0 9.80665	// m/s^2
#define ATM 101325.0	// Pa

/*** USING SI UNITS ***
 * mass: kg
 * weight, thrust: N
 * length: m
 * volume: m^3
 * pressure: Pa
 * speed: m/s
 */

/* TODO:
 * fairing is supposed to be jettisoned, but since there is no trajectory
 * simulation or even delta-V accounting, the gain in net thrust is not taken
 * into account in the mass of the rocket
 * Isp is assumed constant for the same reasons
 */

struct tank_struct {		// used for runtime data
	double i_length; double i_radius; double i_diameter;	// internal volume
	double thickness; double i_volume; double empty_mass;
	double o_length; double o_diameter; double o_surface;
};
typedef struct tank_struct tank;

typedef enum { LOX, RP1, E85, LH2, ETHANOL925 } propellant_type;
struct propellant_struct {		// used for input data
	propellant_type propellant;
	char *name;
	double pressure;
	double tank_thickness;		// force value instead of computing it
	double tank_material_density;
	int is_outer_skin;		// the tank's outer skin is the rocket's
	double insulation_thickness;	// wanted thickness for insulation
	double evaporating_mass;	// before ignition, mass to add to the tank
	double expansion_ratio;		// volume let empty to allow for expansion
	double density;			// set by program
	int is_cryo;			// set by program
};
typedef struct propellant_struct propellant;

void handle_command_line_parameters(int argc, char ** argv);
void display_tank(propellant *prop, tank *t);
void display_parameters();
void display_usage(char *argv0);
void mass_ratio_from_Isp_and_DV();
double get_fuel_ratio(propellant *fuel, propellant *oxidizer);
double prop_mass_to_volume(propellant *prop, double mass);
void initialize_density(propellant *prop);
double evaluate_gravity_drag(double propellant_total_mass, double thrust);

/***************************** INPUT DATA *****************************/

double deltaV = 8300.;		// km/s
//double deltaV = 0.0;		// km/s
double isp = 340.;		// s (assumed constant, V_e is derived from it)

/* ratios */
// WARNING: mass ratio is recomputed if deltaV and isp are non zero
double mass_ratio = 0.0;		// wet mass / dry mass
double engine_thrust_to_weight = 100.0;
double overall_thrust_to_weight = 2.0;	// ignition thrust ratio
double length_to_diameter = 8.0;	// size of the propellant tanks
					// ltd = (h1 + h2) / d, with d = 2r

double payload_mass = 0.0199;		// < 20g for N-prize
double computer_board_mass = 0.1;	// computer board for avionics and telemetry
/* 60g for the board and 40g for the telemetry */
double computer_battery_mass = 0.15;	// bat. for all but thrust vector actuators
double sensors_mass = 0.05;		// 50g should be more than enough
/* accelerometer or gyroscope = 50mg
 * with breakout: 1.4g
 * pressure sensor: 4g (without tubing and pitot)
 * camera: 2g?
 */

/* fuel data. WARNING: it is also required to change the mass ratio or isp when
 * changing propellants. Pressure is used to calculate roughly the tank's wall's
 * thickness 1mm per 10 atmospheres of pressure. */
/*			type, name, p, t_thickn, t_rho, skin, ins_thickn, vap, exp_rat */
propellant oxidizer = { LOX, "LOX", 10.*ATM, .002, 2700., 1,	.01,	1.25, 1.006 };
propellant fuel = {	E85, "E85", 1.*ATM,  .001, 950.,  0,	0.0,	0., 1.002 };
//propellant fuel = {	LH2, "LH2", 10*ATM, .002, 2700., .01,	1.5, 1.005 };

/* aluminium density is 2700 kg/m^3
 * stainless steel density is around 8030 kg/m^3
 * composite carbon/epoxy density is 1600 kg/m^3
 * polypropylene density is 855 kg/m^3
 * HDPE density is 950 kg/m^3
 */

/* Tank insulations examples:
 * PU foam, 22mW/m.K, 24kg/m^3 
 * Aerogel, 15mW/m.K, 13kg/m^3
 */
double insulation_density = 13.;
double pipe_insulation_thickness = 0.01;

double skirt_thickness = 0.001;		// for covered tanks and ullage

/* internal data, should not need to be modified by users */
int number_of_iterations = 1000;
double first_dry_mass_estimation = 20.;	// first iteration value
int quiet = 0;
int making_graph = 0;

int tank_is_structural(propellant *p) {
	return p->tank_material_density >= 2000. && p->tank_thickness > 0.001;
}	// quite a basic evaluation, but for now it's fine

/********************** CODE BEGINNING **************************/

double engine_mass(double required_thrust) {
	return required_thrust / engine_thrust_to_weight / G0;
}

double tank_h_for_volume_and_r(tank *t) {
	double v = t->i_volume, r = t->i_radius;
	 /* h = (v - 4/3 pi r^3) / (PI * r^2)
	  * L = h + 2r */
	return 2.0 * r +
		(v - 4. * M_PI * r*r*r / 3.) / (M_PI * r*r);
}

void tank_data(propellant *oxidizer, tank *t_ox, double ox_volume,
		propellant *fuel, tank *t_fuel, double fuel_volume) {
	/* volume of a tank is PI * r^2 * h  +  4 * PI * r^3 / 3
	 * h = h_fuel + h_ox and volume_fuel / volume_ox = h_fuel / h_ox */
	double surface;
	double propellants_volume = ox_volume + fuel_volume;
	//double total_tanks_height;
	double total_thickness_ox, total_thickness_fuel;

	t_ox->i_volume = ox_volume;
	t_fuel->i_volume = fuel_volume;

	if (oxidizer->tank_thickness > 0.)
		t_ox->thickness = oxidizer->tank_thickness;
	else t_ox->thickness = 0.001 + 0.001 * oxidizer->pressure / (10.*ATM);
	if (fuel->tank_thickness > 0.)
		t_fuel->thickness = fuel->tank_thickness;
	else t_fuel->thickness = 0.001 + 0.001 * fuel->pressure / (10.*ATM);

	/* trying to express r as function of tank thicknesses
	 * ratio_LTD = (L + 2t1 + 2t2) / 2(R + t1)
	 * L = 2ratio * (R + t1) - 2t1 - 2t2
	 * V = -4/3 PI R^3 + PI R^2 (2ratio * (R + t1) - 2t1 - 2t2)
	 *   = -4/3 PI R^3 + 2 PI R^2 (ratio * R + ratio * t1) - 2 PI R^2 t1 - 2 PI R^2 t2
	 *   = -4/3 PI R^3 + 2 PI R^3 ratio + 2 PI R^2 (ratio * t1 - t1 - t2)
	 */
	t_ox->i_radius = cbrt(propellants_volume /
			(2. * M_PI * length_to_diameter - 4. * M_PI / 3.));
	t_fuel->i_radius = t_ox->i_radius;

	/* aligning outer shells */
	// a soft tank is under an outer skirt, its outer diameter thus cannot
	// be the same than a tank with its outer skin actually outside
	total_thickness_ox = t_ox->thickness;
	if (oxidizer->is_cryo) total_thickness_ox += oxidizer->insulation_thickness;
	if (!oxidizer->is_outer_skin) total_thickness_ox += skirt_thickness;
	total_thickness_fuel = t_fuel->thickness;
	if (fuel->is_cryo) total_thickness_fuel += fuel->insulation_thickness;
	if (!fuel->is_outer_skin) total_thickness_fuel += skirt_thickness;
	if (total_thickness_fuel < total_thickness_ox)
		t_fuel->i_radius += (total_thickness_ox - total_thickness_fuel);
	else 	t_ox->i_radius += (total_thickness_fuel - total_thickness_ox);
	t_ox->i_diameter = t_ox->i_radius * 2.;
	t_ox->o_diameter = t_ox->i_diameter + total_thickness_ox * 2.;
	t_fuel->i_diameter = t_fuel->i_radius * 2.;
	t_fuel->o_diameter = t_fuel->i_diameter + total_thickness_fuel * 2.;

	/* calculating tank height using volume formula (for any d) */
	t_ox->i_length = tank_h_for_volume_and_r(t_ox);
	t_fuel->i_length = tank_h_for_volume_and_r(t_fuel);
	t_ox->o_length = t_ox->i_length + total_thickness_ox * 2.;
	t_fuel->o_length = t_fuel->i_length + total_thickness_fuel * 2.;

	/* calculating tank height using direct l/d ratio (requires d given by
	 * the volume formula and identical for both tanks), used in v2 */
	/* total_tanks_height = length_to_diameter * t_ox->diameter;
	t_ox->length = total_tanks_height / (1. + 1. / (ox_volume / fuel_volume));
	t_fuel->length = total_tanks_height - t_ox->length; */

	/* now we need the surface of the tank for weight calculation:
	 * surface of sphere is 4 PI * R^2, of cylinder is 2 PI * R * H
	 * tank surface is thus 2 PI * R (2R + H), with H = L - 2R
	 * so it is 2 PI R L = PI D L
	 */
	surface = M_PI * t_ox->i_diameter * t_ox->i_length;
	t_ox->empty_mass = oxidizer->tank_material_density *
		surface * t_ox->thickness;
	t_ox->empty_mass += insulation_density *
		surface * oxidizer->insulation_thickness;
	/* this is an approximation, the real volume is external diameter
	 * cylinder minus internal diameter cylinder volumes */

	surface = M_PI * t_fuel->i_diameter * t_fuel->i_length;
	t_fuel->empty_mass = fuel->tank_material_density *
		surface * t_fuel->thickness;
	t_fuel->empty_mass += insulation_density *
		surface * fuel->insulation_thickness;
}

double volume_of_a_cylinder(double inner_diameter, double thickness, double length) {
	double r1 = inner_diameter / 2.0;
	double r2 = r1 + thickness;
	return M_PI * (r2 * r2 - r1 * r1) * length;
}

/* pipes and valves mass */
double propellant_equipment_mass(tank *t_ox, tank *t_fuel) {
	// Al tubes for propellant flow to the engine
	// relief valve for cryogenic propellant
	// valves: filling, flow to the engine
	int cryo_tanks = 0;
	double mass = 0.0;
	double ox_pipe_diam = .04, ox_pipe_thickness = .001;
	if (oxidizer.is_cryo) cryo_tanks++;
	if (fuel.is_cryo) cryo_tanks++;
	if (cryo_tanks)
		mass = .1 + (double) cryo_tanks;	// 100g relief valve
	mass += .2 * 2. +	// electromechanical valve mass estimated to 200g
		.1 * 2. +	// manual filling/purging valve to 100g
		// oxidizer line: the length of the fuel tank
		volume_of_a_cylinder(ox_pipe_diam, ox_pipe_thickness,
				t_fuel->o_length+t_fuel->i_radius) * 2700.0;
	// insulation for this pipe
	if (oxidizer.is_cryo)
		mass += volume_of_a_cylinder(ox_pipe_diam+ox_pipe_thickness,
				pipe_insulation_thickness,
				t_fuel->o_length+t_fuel->i_radius) *
			insulation_density;
	return mass;
}

/* rocket main structure, aircraft staging mechanism, avionics empty bay,
 * fairing and its mechanism, inter-tank skirt or external skin */
double structure_mass(tank *t_fuel, tank *t_ox) {
	double mass = 0.0,
	       // structure holding tanks together and tanks to the rest of the structure
	       structure_length = 2.0 * t_ox->o_diameter;
	if (tank_is_structural(&oxidizer))
		mass += .15;	// 150g for fasteners
	else structure_length += t_ox->o_length;
	if (tank_is_structural(&fuel))
		mass += .15;	// 150g for fasteners
	else structure_length += t_fuel->o_length;
	// main structure is 5 rods, 2mm thick Al, 15mm wide
	mass = .002 * .015 * structure_length * 5. * 2700.0;
	mass += .15;	// aircraft holers and staging mechanism
	// skirts as composite cylinders, 10cm spare for engine and bay
	mass += (2. * t_ox->o_diameter + 0.1) * M_PI * t_ox->o_diameter *
		1600. * skirt_thickness;
	return mass;
}

double fairing_mass(double vehicle_width) {
	// fairing assimilated as a 2mm-thick cone of length=2D, and mechanism for 100g
	// area of a cone is PI * r * s, r being the radius and s the slant height
	double r = vehicle_width / 2.0;
	double area = M_PI * r * sqrt(r*r + 4.0*vehicle_width*vehicle_width);
	return area * .002 * 2700. + .1;
	/* NOTE: fairing could be made much lighter with another material. With
	 * an air launch, there is much less atmospheric friction, it won't heat
	 * as much as with a ground launch. */
}

double wiring_mass(double vehicle_length) {
	return vehicle_length * .07;
}

/* mass for the thrust vector actuators (i.e. servo mechanism or hydraulic
 * system) + battery required for the chosen system. */
double actuators_mass(double engine_thrust) {
	return engine_thrust / 8000. +		// servo actuators
		engine_thrust / 15000. ;	// battery
}

void iterate() {
	int i=0;
	double thrust, propellant_mass, fuel_mass, oxidizer_mass;
	double fuel_volume, oxidizer_volume, vehicle_length;
	double m_engine, m_actuators, m_wiring, m_structure, m_pipes, m_fairing;
	tank tank_fuel, tank_oxidizer;
	double vehicle_dry_mass, vehicle_wet_mass, fuel_ratio;

	mass_ratio_from_Isp_and_DV();
	vehicle_dry_mass = first_dry_mass_estimation;	
#ifndef ITERATE_ON_DRY
	vehicle_wet_mass = vehicle_dry_mass * mass_ratio;
#endif
	/* init data */
	fuel_ratio = get_fuel_ratio(&fuel, &oxidizer);
	initialize_density(&oxidizer);
	initialize_density(&fuel);

	do {
#ifdef ITERATE_ON_DRY
		double next_dry_mass;
		vehicle_wet_mass = vehicle_dry_mass * mass_ratio;
#else
		double next_wet_mass;
		vehicle_dry_mass = vehicle_wet_mass / mass_ratio;
#endif
		propellant_mass = vehicle_wet_mass - vehicle_dry_mass;
		thrust = vehicle_wet_mass * G0 * overall_thrust_to_weight;
		/* fuel_ratio = ox mass / fuel mass
		 * prop mass = fuel mass + ox mass
		 * prop mass = fuel mass + fuel mass * stoich ratio
		 * prop mass = fuel mass ( 1 + stoich ratio )
		 * fuel mass = prop mass / ( 1 + stoich ratio )
		 *
		 * prop mass = ox mass / stoich ratio + ox mass
		 * prop mass = ox mass ( 1 + 1 / stoich ratio )
		 * ox mass = prop mass / (1 + 1 / stoich ratio )
		 */
		fuel_mass = propellant_mass / (1. + fuel_ratio);
		fuel_mass += fuel.evaporating_mass;
		oxidizer_mass = propellant_mass / (1. + 1. / fuel_ratio);
		oxidizer_mass += oxidizer.evaporating_mass;
		//fprintf(stdout,"fuel mass: %g, oxidizer mass: %g, propellant mass: %g\n",
		//		fuel_mass, oxidizer_mass, propellant_mass);
		fuel_volume = prop_mass_to_volume(&fuel, fuel_mass);
		fuel_volume *= fuel.expansion_ratio;
		oxidizer_volume = prop_mass_to_volume(&oxidizer, oxidizer_mass);
		oxidizer_volume *= oxidizer.expansion_ratio;
		tank_data(&oxidizer, &tank_oxidizer, oxidizer_volume,
				&fuel, &tank_fuel, fuel_volume);
		vehicle_length = tank_fuel.o_length + tank_oxidizer.o_length; // roughly

		m_engine = engine_mass(thrust);
		m_actuators = actuators_mass(thrust);
		m_wiring = wiring_mass(vehicle_length);
		m_structure = structure_mass(&tank_fuel, &tank_oxidizer);
		m_pipes = propellant_equipment_mass(&tank_oxidizer, &tank_fuel);
		m_fairing = fairing_mass(tank_fuel.o_diameter);

#ifdef ITERATE_ON_DRY
		next_dry_mass =
#else
		next_wet_mass = propellant_mass +
#endif
			/* fixed stuff */
			payload_mass + computer_board_mass + sensors_mass +
			computer_battery_mass +
			/* variable stuff */
			tank_fuel.empty_mass + tank_oxidizer.empty_mass +
			m_engine + m_actuators + m_wiring + m_structure +
			m_pipes + m_fairing;

#ifdef ITERATE_ON_DRY
#if DEBUG
		fprintf(stdout, "DRY MASS: %g kg, vehicle size: %g x %g m\n",
				next_dry_mass, vehicle_length, tank_fuel.diameter);
#endif
		vehicle_dry_mass = next_dry_mass;
#else
#if DEBUG
		fprintf(stdout, "DRY MASS: %g kg, tanks size: %g x %g m\n",
				vehicle_dry_mass, vehicle_length, tank_fuel.diameter);
#endif
		vehicle_wet_mass = next_wet_mass;
#endif
		i++;
	} while (i < number_of_iterations) ;

	/* WARNING: vehicle_wet_mass is the mass of the dry vehicle + propellant,
	 * it does not include additional propellant mass that will be evaporated. */
	//vehicle_wet_mass += fuel.evaporating_mass + oxidizer.evaporating_mass;
	if (making_graph) {
		/* print out the iterated parameter and result for graph data */
		fprintf(stdout, "%g\t%g\t%g\n",
				isp, 
				vehicle_dry_mass,
				vehicle_wet_mass);
	} else {
		fprintf(stdout, "WET MASS: %g kg, tanks size: %g x %g m\n",
				vehicle_wet_mass, vehicle_length, tank_fuel.o_diameter);
		fprintf(stdout, "DRY mass: %g kg, fairing mass: %g kg\n",
				vehicle_dry_mass, m_fairing);
	}
	if (!quiet) {
		fprintf(stdout, "\nEngine thrust: %g N, engine mass: %g kg\n", thrust, m_engine);
		fprintf(stdout, "Final thrust-to-weight-ratio: %g G\n",
				thrust / ((vehicle_dry_mass - m_fairing) * G0));
		fprintf(stdout, "Actuators mass: %g kg, wiring mass: %g kg\n"
				"Structure mass: %g kg, valves and pipes: %g kg\n",
				m_actuators, m_wiring, m_structure, m_pipes);
		fprintf(stdout,"Ground fuel mass: %g kg, oxidizer mass: %g kg\n",
				fuel_mass, oxidizer_mass);
		/* ^ these are the actual lift-off values, evaporated mass included ^ */
		display_tank(&oxidizer, &tank_oxidizer);
		display_tank(&fuel, &tank_fuel);
		fputc('\n', stdout);
	}
	evaluate_gravity_drag(propellant_mass, thrust);
}

double evaluate_gravity_drag(double propellant_total_mass, double thrust) {
	double mfr, burn_time, dg;
	mfr = thrust / (G0 * isp);
	burn_time = propellant_total_mass / mfr;
	dg = burn_time * 8.5;
	fprintf(stdout, "burn time: %g, gravity drag: %g\n", burn_time, dg);
	return dg;
}

int main(int argc, char **argv) {
	handle_command_line_parameters(argc, argv);
	mass_ratio_from_Isp_and_DV();	// by default, recompute it
#ifdef MAKE_GRAPH
	making_graph = 1;	quiet = 1;
#endif
	if (!quiet) {
		display_parameters();
#ifdef ITERATE_ON_DRY
		fprintf(stdout, "%d iterations on dry mass\n",
				number_of_iterations);
#else
		fprintf(stdout, "%d iterations on wet mass\n",
				number_of_iterations);
#endif
	}
	
#ifdef MAKE_GRAPH
	/* print a comment for the data output, indicating what data is about */
	//fprintf(stdout, "# overall thrust_to_weight\tthrust\tfinal acceleration\n");
	/* iterate over a parameter. Change it here if needed */
	for (isp = 280.; isp <= 450.; isp += 10.)
#endif
		iterate();
	return 0;
}

void display_tank(propellant *prop, tank *t) {
	fprintf(stdout, "%s tank info:\n", prop->name);
	fprintf(stdout, "\tdiameter: %g m, length: %g m\n\twall thickness: %g m, "
			"volume: %g m^3, mass: %g kg\n",
			t->o_diameter, t->o_length, t->thickness,
			t->i_volume, t->empty_mass);
}

void display_parameters() {
	fprintf(stdout, "----------------- Input parameters -----------------\n");
	if (deltaV > 0. && isp > 0.)
		fprintf(stdout, "Computed mass ratio of %g"
				" (from Isp=%g and DeltaV=%g)\n",
				mass_ratio, isp, deltaV);
	else fprintf(stdout, "Using mass ratio of %g\n", mass_ratio);
	fprintf(stdout, "Propellant is %g%% of the rocket's mass.\n",
			(1.-1./mass_ratio)*100.);
	fprintf(stdout, "Length-to-diameter ratio (tanks): %g\n",
			length_to_diameter);
	fprintf(stdout, "Engine thrust-to-weight ratio: %g. "
			"Overall thrust-to-weight ratio: %g\n",
			engine_thrust_to_weight, overall_thrust_to_weight);
	fprintf(stdout, "Payload mass: %g kg. Computer fixed mass: %g kg. "
			"Sensors mass: %g kg\n",
			payload_mass, computer_board_mass+computer_battery_mass,
			sensors_mass);
	fprintf(stdout, "----------------------------------------------------\n");
}

/************************* PROPELLANT DATA ****************************/

/* O/F ratio by mass. This is not the stoichiometric ratio, but the more
 * efficient ratio, giving the best Isp or with benefits for density */
double get_fuel_ratio(propellant *fuel, propellant *oxidizer) {
	switch (oxidizer->propellant) {
		case LOX:
			switch (fuel->propellant) {
				case RP1: return 2.56;
				case E85: return 2.26;
				case LH2: return 5.0;	// stoich is 8
				case ETHANOL925: return 2.19;
				default: break;
			}
		default: break;
	}
	fprintf(stderr, "O/F ratio unknow for selected fuels\n");
	exit(1);
}

double prop_mass_to_volume(propellant *prop, double mass) {
	return mass / prop->density;
}

/* density is assumed constant, or at 1 atm boiling temperature if it applies */
void initialize_density(propellant *prop) {
	switch (prop->propellant) {
		case LOX: prop->density = 1140.; prop->is_cryo = 1; return;
		case RP1: prop->density = 806.; return;
		case E85: prop->density = 780.; return;
		case LH2: prop->density = 71.; prop->is_cryo = 1; return;
		case ETHANOL925: prop->density = 800.; return;
	}
	fprintf(stderr, "unknown propellant used %d\n", prop->propellant);
	exit(1);
}

/* depending on mission (Delta-V) and engine parameters (chosen fuel, chamber
 * pressure, engine nozzle, giving exhaust velocity), the mass ratio of the
 * rocket is given by the rocket equation:
 * e^(-delta_v/exhaust_velocity) is dry mass / wet mass
 *
 * the exhaust velocity can also be obtained from Isp, the relation between them
 * being: Isp * G0 = exhaust velocity. G0 is the standard gravity constant.
 *
 * so the mass ratio from the Isp is:
 * 1 / e^(-Delta V / (Isp * G0))	with delta V in m/s instead of km/s
 */
void mass_ratio_from_Isp_and_DV() {
	if (deltaV > 0.0 && isp > 0.0)
		mass_ratio = 1. / exp(- deltaV / (isp * G0));
}

/************************* COMMAND LINE ****************************/

void handle_command_line_parameters(int argc, char ** argv) {
	int i;
	if (argc <= 1) return;
	if (!strcmp(argv[1], "-h"))
		display_usage(*argv);
	if (!strcmp(argv[1], "-q")) {
		quiet = 1;
		i = 2;
		if (argc == 2) return;
	} else 	i = 1;
	if (argc - i <= 1) display_usage(*argv);
	do {
		double val = atof(argv[i+1]);
		if (val == 0.0) display_usage(*argv);
		switch (argv[i][1]) {
			case '\0':
				break;
			case 'r':
				mass_ratio = val;
				break;
			case 'e':
				engine_thrust_to_weight = val;
				break;
			case 'o':
				overall_thrust_to_weight = val;
				break;
			case 'l':
				length_to_diameter = val;
				break;
			case 'p':
				payload_mass = val;
				break;
			case 'c':
				computer_board_mass = val;
				break;
			case 'i':
				isp = val;
				break;
			case 'd':
				deltaV = val;
				break;
			default:
				fprintf(stderr, "Unhandled option %s\n", argv[i]);
				display_usage(*argv);
		}
		i+=2;
	} while (i < argc-1);
}

void display_usage(char *argv0) {
	fprintf(stderr, "Usage: %s [-r mass_ratio] [-d delta_v] [-i Isp]\n"
			"\t[-e engine_thrust_to_weight] [-o overall_thrust_to_weight]\n"
			"\t[-l length_to_diameter] [-p payload_mass] [-c computer_mass]\n",
			argv0);
	exit(1);
}

